/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            fow.c
 *
 *  Fri Oct 14 22:00:43 CEST 2011
 *  Copyright 2011 Jonas Suhr Christensen
 *  jsc@umbraculum.org
 ****************************************************************************/

/*
 *  This file is part of xboard-fow.
 *
 *  Glpr is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Glpr is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Glpr; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

#include "fow.h"

static int playersSquare(ChessSquare piece) {
  int r;
  if(gameMode == 0) r = 1;
  else if(gameMode == MachinePlaysBlack && White(piece)) r = 1;
  else if(gameMode == MachinePlaysWhite && Black(piece)) r = 1;
  else if(gameMode == IcsPlayingWhite && White(piece)) r = 1;
  else if(gameMode == IcsPlayingBlack && Black(piece)) r = 1;
  else r = 0;

  return r;
}

void hideAllSquares(int(*b)[BOARD_WIDTH]) {
  int i, j;
  // hide all squares in visible board
  for(i = 0; i < BOARD_HEIGHT; i++) {
    for(j = 0; j < BOARD_WIDTH; j++) {
      b[i][j] = 0;
    }
  }
}

static int insideBoard(int i, int j) {
  if(i < BOARD_HEIGHT && i > 0 &&
     j < BOARD_WIDTH && j > 0) return 1;
  else return 0;
}

static void visibleFromPawn(int i, int j, ChessSquare p, int(*b)[BOARD_WIDTH]) {
  if(White(p) && insideBoard(i-1, j)) b[i-1][j] = 1;
  else if(insideBoard(i+1, j)) b[i+1][j] = 1;
}

static void visibleFromKnight(int i, int j, ChessSquare p, int(*b)[BOARD_WIDTH]) {
  // South 
  if( insideBoard(i+1,j) ) b[i+1][j] = 1;
  if( insideBoard(i+2,j) ) b[i+2][j] = 1;
  if( insideBoard(i+2,j+1) ) b[i+3][j+1] = 1;
  if( insideBoard(i+2,j-1) ) b[i+3][j-1] = 1;

  // North
  if( insideBoard(i-1,j) ) b[i-1][j] = 1;
  if( insideBoard(i-2,j) ) b[i-2][j] = 1;
  if( insideBoard(i-2,j+1) ) b[i-3][j+1] = 1;
  if( insideBoard(i-2,j-1) ) b[i-3][j-1] = 1;

  // West
  if( insideBoard(i,j-1) ) b[i][j-1] = 1;
  if( insideBoard(i,j-2) ) b[2][j-2] = 1;
  if( insideBoard(i-1,j-2) ) b[i-1][j-3] = 1;
  if( insideBoard(i+1,j-2) ) b[i+1][j+3] = 1;

  // East
  if( insideBoard(i,j+1) ) b[i][j+1] = 1;
  if( insideBoard(i,j+2) ) b[2][j+2] = 1;
  if( insideBoard(i-1,j+2) ) b[i-1][j+3] = 1;
  if( insideBoard(i+1,j+2) ) b[i+1][j+3] = 1;
}

static void visibleFromBishop(int i, int j, ChessSquare p, int(*b)[BOARD_WIDTH]) {
  int it_i, it_j;

  // South East
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_i++; it_j++; }
  
  // South West
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_i++; it_j--; }

  // North West
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_i--; it_j--; }

  // North West
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_i--; it_j++; }
}

static void visibleFromRook(int i, int j, ChessSquare p, int(*b)[BOARD_WIDTH]) {
   int it_i, it_j;

  // South
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_i++; }
  
  // North
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_i--; }

  // West
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_j--; }

  // East
  it_i = i; it_j = j;
  while( insideBoard(it_i,it_j) ) { b[it_i][it_j] = 1; it_j++; } 
}

static void visibleFromQueen(int i, int j, ChessSquare p, int(*b)[BOARD_WIDTH]) {
  visibleFromRook(i, j, p, b);
  visibleFromBishop(i, j, p, b);
}

static void visibleFromKing(int i, int j, ChessSquare p, int(*b)[BOARD_WIDTH]) {
  // South
  if( insideBoard(i+1, j) ) b[i+1][j] = 1;

  // North 
  if( insideBoard(i-1, j) ) b[i-1][j] = 1;
  
  // West
  if( insideBoard(i, j-1) ) b[i][j-1] = 1;

  // East 
  if( insideBoard(i, j+1) ) b[i][j+1] = 1;
}

void compVisibleBoard(Board board, int(*vb)[BOARD_WIDTH]) {
  int i, j;

  hideAllSquares(vb); 

  //calculate visible fields for each square
  for(i = 0; i < BOARD_HEIGHT; i++) {
    for(j = 0; j < BOARD_WIDTH; j++) {
      ChessSquare piece = board[i][j];
      
      if(!playersSquare(piece)) continue;
      else vb[i][j] = 1; 

      switch(piece) {
        case WhitePawn:
	case BlackPawn:
	  visibleFromPawn(i, j, piece, vb);
	  break;
        case WhiteKnight:
        case BlackKnight:
	  visibleFromKnight(i, j, piece, vb);
          break;
        case WhiteBishop:
        case BlackBishop:
	  visibleFromBishop(i, j, piece, vb);
	  break;
        case WhiteRook:
        case BlackRook:
	  visibleFromRook(i, j, piece, vb);
	  break;
        case WhiteQueen:
        case BlackQueen:
	  visibleFromQueen(i, j, piece, vb);
	  break;
        case WhiteKing:
        case BlackKing:
	  visibleFromKing(i, j, piece, vb);
          break;
        default:
          break;
      }
    }
  }
}
